/****************************************************************************\

PSOProxy - http proxy server for the PSO web browser

Copyright (C) 2003 Adrian O'Grady and James Sutherland

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

\****************************************************************************/

// HttpHeader.cpp: implementation of the CHttpHeader class.
//
//////////////////////////////////////////////////////////////////////

#include "string.h"
#include "HttpHeader.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHttpHeader::CHttpHeader()
{
	m_Head = 0;
	m_Method[0] = 0;
	m_Version[0] = 0;
}

CHttpHeader::~CHttpHeader()
{
	if (m_Head) DestroyChain(m_Head);
}

void CHttpHeader::AddHeader(const char *Header)
{
	char * key = new char[strlen(Header)+1];
	char * value = 0;

	strcpy(key, Header);

	value = strstr(key, ": ");
	if (value)
	{
		*value = 0;
		value += 2;
		AddPair(key, value, m_Head);
	}

	delete[] key;
}

void CHttpHeader::AddPair(const char *Key, const char *Value, kvpair *& Chain)
{
	if (Chain)
	{
		AddPair(Key, Value, Chain->Next);
	}
	else
	{
		Chain = new kvpair;
		strcpy(Chain->Key, Key);
		strcpy(Chain->Value, Value);
		Chain->Next = 0;
	}
}

void CHttpHeader::DestroyChain(kvpair * Chain)
{
	if (Chain->Next)
	{
		DestroyChain(Chain->Next);
	}
	delete Chain;
}

const char * CHttpHeader::GetValue(const char *Key)
{
	return GetValue(Key, m_Head);
}

const char * CHttpHeader::GetValue(const char * key, kvpair *Chain)
{
	if (Chain)
	{
		if (strcmp(Chain->Key, key) == 0)
		{
			return Chain->Value;
		}
		else
		{
			return GetValue(key, Chain->Next);
		}
	}
	else
	{
		return 0;
	}
}

bool CHttpHeader::StartRequest(const char *Request)
{
	bool r = false;
	char * requestBuffer = new char[strlen(Request)+1];
	char * Found = 0;
	char * NextWordStart = 0;
	
	strcpy(requestBuffer, Request);

	// Find the first word
	Found = strstr(requestBuffer, " ");
	if (Found)
	{
		*Found = 0;
		Found++;
		strcpy(m_Method, requestBuffer);

		NextWordStart = Found;

		Found = strstr(NextWordStart, " ");
		if (Found)
		{
			*Found = 0;
			Found++;
			
			m_URL.ParseURL(NextWordStart);
			strcpy(m_Version, Found);

			r = true;
		}
	}

	delete[] requestBuffer;

	return r;
}

const char * CHttpHeader::GetProtocol()
{
	return m_URL.GetProtocol();
}

const char * CHttpHeader::GetHost()
{
	return m_URL.GetHost();
}

unsigned short CHttpHeader::GetPort()
{
	return m_URL.GetPort();
}

const char * CHttpHeader::GetPath()
{
	return m_URL.GetPath();
}

const char * CHttpHeader::GetExtra()
{
	return m_URL.GetExtra();
}

const char * CHttpHeader::GetMethod()
{
	return m_Method;
}

const char * CHttpHeader::GetVersion()
{
	return m_Version;
}
