// Interfaces.h: A list of interfaces used by the proxy and its applets.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_IPROXYAPI_H__F23D67BD_434E_41B9_9CF4_88B6D2C4C2A2__INCLUDED_)
#define AFX_IPROXYAPI_H__F23D67BD_434E_41B9_9CF4_88B6D2C4C2A2__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// Forward defines
class IHttpHeader;
class ISettingsEnumerator;

// Proxy APIs
class IProxyAPI1
{
public:
	virtual int				Send(const char *Data, int Length) = 0;
	virtual int				Recv(char *Data, int Length) = 0;
	
	virtual void			SendHeader(const char *Result, const char *ContentType, unsigned int ContentLength, const char *Extra) = 0;
	virtual void			SendInternalError(const char *Error) = 0;
	virtual bool			SendPage(const char *Filename) = 0;
	virtual bool			SendDynamicPage(const char *Filename) = 0;
	virtual IHttpHeader&	GetHttpHeader() = 0;

	virtual int				ExecApplet(const char *AppletName, const char *Args) = 0;

	virtual const char *	GetSettingAsString(const char *Key, const char *Default) = 0;
	virtual int				GetSettingAsInt   (const char *Key, int Default) = 0;
	virtual float			GetSettingAsFloat (const char *Key, float Default) = 0;
	virtual void			GetAllSettings    (ISettingsEnumerator *callback) = 0;
	virtual void			SetSetting        (const char *Key, const char *Value) = 0;
	virtual void			SetSetting        (const char *Key, int Value) = 0;
	virtual void			SetSetting        (const char *Key, float Value) = 0;

	virtual void			Log(const char *Text) = 0;

	virtual const char *	GetServerVersionString() = 0;
	virtual void			GetServerVersion(int& Major, int& Minor, int& Revision) = 0;
};

// Applet API
class IProxyApplet1
{
public:
	virtual int				appmain(unsigned int argc, const char** argv) = 0;
	virtual void			terminate() = 0;

	virtual void			OnProxyTag(int TagNumber) {}
};


// Header interface
class IHttpHeader
{
public:
	virtual const char *	GetMethod() = 0;
	virtual const char *	GetProtocol() = 0;
	virtual const char *	GetHost() = 0;
	virtual unsigned short	GetPort() = 0;
	virtual const char *	GetPath() = 0;
	virtual const char *	GetExtra() = 0;
	virtual const char *	GetVersion() = 0;

	virtual const char *	GetValue(const char *Key) = 0;
};

// Settings Enumerator
class ISettingsEnumerator
{
public:
	virtual void OnDataItem(const char * Key, const char * Value, bool LastItem) = 0;
};
#endif // !defined(AFX_IPROXYAPI_H__F23D67BD_434E_41B9_9CF4_88B6D2C4C2A2__INCLUDED_)
