/****************************************************************************\

PSOProxy - http proxy server for the PSO web browser

Copyright (C) 2003 Adrian O'Grady and James Sutherland

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

\****************************************************************************/

// PageServer.cpp: implementation of the PageServer class.
//
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "PageServer.h"
#include "Settings.h"
#include "MimeDB.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPageServer::CPageServer(IProxyAPI1& API) : m_API(API)
{

}

CPageServer::~CPageServer()
{

}

// All the processing happens here as the applet isn't expecting any more data
int CPageServer::appmain(unsigned int argc, const char **argv)
{
	char Path[256];
	char Result[32];

	fpos_t FileLen = 0;
	FILE * file = 0;
	Result[0] = 0;

	char LogString[512];		// Used for log output


	// Were there any extra args?
	if (argc == 0)
	{
		// Log what has been requested
		sprintf(LogString, "%s requested", m_API.GetHttpHeader().GetPath());
		m_API.Log(LogString);

		// Start in the default directory
		strcpy(Path, m_API.GetSettingAsString("webroot", "webroot/"));
		if (Path[strlen(Path)-1] != '/')
		{
			strcat(Path, "/");
		}

		// How much do we need to fix/fudge the path?
		if (m_API.GetHttpHeader().GetPath()[0] == '/')
		{
			if (strlen(m_API.GetHttpHeader().GetPath()) == 1)
			{
				// Not a full path, so serve the default page
				strcat(Path, "index.html");
			}
			else
			{
				// If the first character is a '/' we already have one. If you see what I mean
				strcat(Path, m_API.GetHttpHeader().GetPath()+1);
			}
		}
		else
		{
			// We have a path that doesn't start with '/', stick it on the path for the final name
			strcpy(Path, m_API.GetHttpHeader().GetPath());
		}
	}
	else
	{
		// The header has been over ridden by an argument
		strcpy(Path, argv[0]);

		// Was there a second arg?
		if (argc > 1)
		{
			// Yes, used the supplied result
			strcpy(Result, argv[1]);
		}
		else
		{
			// No, default to a safe(ish) one
			strcpy(Result, "200 OK");
		}

	}

	// Log what is attempted to be served
	sprintf(LogString, "Serving %s", Path);
	m_API.Log(LogString);

	// Now to start opening the file we're serving
	file = fopen(Path, "rb");
	if (file == NULL)
	{
		// Log what we've been told to serve
		sprintf(LogString, "%s was not found", Path);
		m_API.Log(LogString);

		// Couldn't open the file, so it wasn't found
		strcpy(Path, m_API.GetSettingAsString("404page", "webroot/404.html"));
		file = fopen(Path, "rb");
		strcpy(Result, "404 Not Found");

		if (file == NULL)
		{
			m_API.Log("!!! 404page not found !!!");

			m_API.SendInternalError("404page not found.<br>The server is incorrectly installed.");

			// Bomb out.
			return 1;
		}
	}
	else
	{
		// File's found so set the result to OK
		if (Result[0] == 0) strcpy(Result, "200 OK");
	}

	// I might have cocked up, so who knows
	assert(file);

	// Get the length of the file
	fseek(file, 0, SEEK_END);
	fgetpos(file, &FileLen);
	fseek(file, 0, SEEK_SET);
	// Close the file
	fclose(file);

	// Send a http header to the client
	
	m_API.SendHeader(Result, CMimeDB::Get().GetType(Path),	(unsigned int)FileLen, 0);
	m_API.SendPage(Path);
	
	// The applet executed
	return 0;
}

IProxyApplet1 * CPageServer::CreateApplet(IProxyAPI1& API)
{
	return new CPageServer(API);
}

void CPageServer::ReleaseApplet(IProxyApplet1 *Applet)
{
	delete Applet;
}

void CPageServer::terminate()
{

}
