/****************************************************************************\

PSOProxy - http proxy server for the PSO web browser

Copyright (C) 2003 Adrian O'Grady and James Sutherland

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

\****************************************************************************/

// RequestHandler.cpp: implementation of the CRequestHandler class.
//
//////////////////////////////////////////////////////////////////////

#include <string.h>
#include "RequestHandler.h"
#include "AppletDB.h"
#include "Settings.h"
#include "VirtualPathMan.h"

#include "PageServer.h"
#include "SaveFile.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRequestHandler::CRequestHandler() : m_Input(3500)
{
	m_state = AWAITING_REQUEST;
}

CRequestHandler::~CRequestHandler()
{

}

CRequestHandler::eAction CRequestHandler::StreamIn(const char *Data, unsigned int Length, int socket)
{
	eAction r = ACT_KEEP_OPEN;
	char * LineData = 0;
	char buffer[1024];
	bool ProcessInput = true;

	// If we're still dealing with the header, run it through the funny stream to line thing
	if ((m_state == AWAITING_REQUEST) || (m_state == READING_HEADER))
	{
		m_Input.StreamIn(Data, Length);
	}

	// We want to keep looping round while we have data. It's only polite
	while (ProcessInput)
	{

		// Nightmare state machine from hell (well, not that bad, just a mess)
		switch (m_state)
		{
		case AWAITING_REQUEST:
			LineData = m_Input.GetLine(buffer);
			
			if (LineData)
			{
				// Start off the request
				if (m_Header.StartRequest(LineData))
				{
					// The request seems valid so far
					m_state = READING_HEADER;
				}
				else
				{
					// No idea what we got, close it all down
					ProcessInput = false;
					r = ACT_CLOSE;
				}
			}
			else
			{
				// No more data
				ProcessInput = false;
			}
			break;

		case READING_HEADER:
			// Now we're reading in the header values
			LineData = m_Input.GetLine(buffer);
			if (LineData)
			{
				// We've read a line
				if (LineData[0] != 0)
				{
					// Add it to our header
					m_Header.AddHeader(LineData);
				}
				else
				{
					// An empty line, time to start executing the request
					m_state = EXEC;
				}
			}
			else
			{
				// No data, time to hand back to the web server
				ProcessInput = false;
			}
			break;

		case EXEC:
			// Prepare the args
			CAppletDB::AppletArgs Args;
			Args.Socket = socket;
			Args.Header = &m_Header;
			Args.BufferedInputLength = strlen(m_Input.GetBuffer());
			Args.BufferedInput = m_Input.GetBuffer();

			char Path[256];
			CVirtualPathMan::Get().GetRealPath(m_Header.GetPath(), Path);
			
			// Decide what we're doing now
			if (CAppletDB::Get().IsApplet(Path))
			{
				// ** TO DO: Convert the Extra header info into args
				Args.argc = 0;
				Args.argv = 0;

				// Create and terminate the new applet
				CAppletDB::Get().ExecApplet(Path, Args);				

				r = ACT_CLOSE;
			}
			else
			{
				Args.argc = 1;
				Args.argv = new const char *;
				Args.argv[0] = Path;

				// Create and terminate the new applet
				CAppletDB::Get().ExecApplet("server", Args);

				delete Args.argv;

				r = ACT_CLOSE;
			}

			// We can assume that all the input has been consumed by executing the request
			ProcessInput = false;

			break;
		}

	}

	return r;
}
