/****************************************************************************\

PSOProxy - http proxy server for the PSO web browser

Copyright (C) 2003 Adrian O'Grady and James Sutherland

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

\****************************************************************************/

// SaveFile.cpp: implementation of the CSaveFile class.
//
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <winsock.h>
#include "stdlib.h"
#include "SaveFile.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSaveFile::CSaveFile(IProxyAPI1& API) : m_API(API)
{
	m_ContentLength = 0;
	m_Saved = 0;
	m_Data = 0;
}

CSaveFile::~CSaveFile()
{

}

// Application entry point
int CSaveFile::appmain(unsigned int argc, const char **argv)
{
	const char * cl = m_API.GetHttpHeader().GetValue("Content-Length");

	// Find out how much data we're dealing with approx
	if (cl && cl[0] != 0)
	{
		m_ContentLength = atoi(cl);
		m_Saved = 0;
		// Allocate memory for the uploaded data (with extra space for a zero)
		m_Data = new char[m_ContentLength + 4];

		ScanForFile();

		return 0;
	}
	else
	{
		m_API.Log("Did not get Content-Length header.");
		return -1;
	}
}

void CSaveFile::SendThankYouPage()
{
	// This just sends a file to the client
	m_API.SendHeader("200 OK", "text/html", 0, 0);
	m_API.SendDynamicPage("webroot/thanks.html");

}

void CSaveFile::ScanForFile()
{
	char Data[1500];
	int Length = 0;

	do
	{
		Length = m_API.Recv(Data, 1500);

		if (Length > 0)
		{
			// Copy the data into memory
			if ((m_Saved + Length) > m_ContentLength)
			{
				Length = m_ContentLength - m_Saved;
			}
			memcpy(&m_Data[m_Saved], Data, Length);
			m_Saved += Length;

			// Have we got all the data?
			if (m_Saved >= m_ContentLength)
			{
				// Two vars to locate the file and then the end of it
				char * Found = 0;
				char * End = 0;

				// Do we can drop out of thh loop
				Length = 0;
				// Close the string
				m_Data[m_Saved] = 0;

				// Is it a gc file?
				Found = strstr(m_Data, "Content-Disposition: form-data; name=\"gcfile\"\x0D\x0A");
				if (Found)
				{
					m_API.Log("Extracting gcfile");

					// Skip the next two lines to cut direct to the data
					Found = strstr(Found, "\x0D\x0A");
					Found = strstr(Found+2, "\x0D\x0A");
					Found += 4;

					End = strstr(Found, "\x0D\x0A");
					// Calculate the length of the file
					if (End)
					{
						Length = End - Found;
					}
					else
					{
						Length = strlen(Found);
					}

					m_ProcessData.OnData(Found, Length);

					m_API.Log("Saved snapshot.");
				}
				/*else
				{
					m_API.Log("Non-gcfile saved as out.txt");

					// Not a gc file, just save it to disk
					FILE * file = fopen("out.txt", "wb");
					fwrite(m_Data, 1, m_Saved, file);
					fclose(file);

					m_API.Log("Saved non gcfile.");
				}*/
				

				SendThankYouPage();

				// Free the memory used by the data file
				delete[] m_Data;
				Length = 0;
			}

		} // if (Length > 0)

	} while (Length > 0);
}

void CSaveFile::terminate()
{

}

void CSaveFile::ReleaseApplet(IProxyApplet1 *Applet)
{
	delete Applet;
}

IProxyApplet1* CSaveFile::CreateApplet(IProxyAPI1& API)
{
	return new CSaveFile(API);
}

void CSaveFile::OnProxyTag(int TagNumber)
{
	char imgTag[256];

	if (m_ProcessData.GetFileName()[0] != 0)
	{
		sprintf(imgTag, "<img src=\"%s%s\"><br>\nUpload successful",
							m_API.GetSettingAsString("vdir", "/snapshots/"),
							m_ProcessData.GetFileName());
	}
	else
	{
		strcpy(imgTag, "No Image");
	}
	m_API.Send(imgTag, strlen(imgTag));
}
