/****************************************************************************\

PSOProxy - http proxy server for the PSO web browser

Copyright (C) 2003 Adrian O'Grady and James Sutherland

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

\****************************************************************************/

// Settings.h: interface for the CSettings class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SETTINGS_H__C166045D_B3B7_41B0_BB90_D76C37807644__INCLUDED_)
#define AFX_SETTINGS_H__C166045D_B3B7_41B0_BB90_D76C37807644__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <fstream.h>
#include "Interfaces.h"

class CSettings
{
public:
	void			GetAllSettings(ISettingsEnumerator *, const char * Section);

	const char *	GetValueAsString(const char * Section, const char * Key, const char * Default);
	int				GetValueAsInt   (const char * Section, const char * Key, int Default);
	float			GetValueAsFloat	(const char * Section, const char * Key, float Default);
	const char *	GetValueAsString(const char * Section, const char * Key);
	int				GetValueAsInt   (const char * Section, const char * Key);
	float			GetValueAsFloat	(const char * Section, const char * Key);

	void			SetValue(const char * Section, const char * Key, const char * Value);
	void			SetValue(const char * Section, const char * Key, int Value);
	void			SetValue(const char * Section, const char * Key, float Value);

	void			SaveIni();
	bool			LoadIni(const char * Filename);

	static CSettings& Get()		{ return s_inst; }
	virtual ~CSettings();

private:
	typedef enum
	{
		BLANK = 0,
		COMMENT,
		SECTION,
		DATA
	} eLineType;

	struct sKVPair
	{
		char * Key;
		char * Value;
		sKVPair * Next;
	};

	struct sSection
	{
		char * Name;
		sKVPair * Data;
		sSection * Next;
	};

	static CSettings	s_inst;
	sSection *			m_Sections;
	char				m_Filename[256];
	bool				m_IsDirty;

private:
	CSettings();
	void			SaveData(sKVPair * Head, ofstream &file);
	void			SaveSection(sSection * Section, ofstream& file);
	void			EnumerateData(ISettingsEnumerator * callback, sKVPair * Pair);
	sKVPair *		FindKey(const char *Key, sKVPair *Head);
	sSection *		FindSection(const char * Name, sSection *Head);
	void			AddData(const char * Key, const char * Value, sKVPair *& Head);
	void			DestroySections(sSection * Head);
	void			DestroyData(sKVPair * Head);
	sSection *		AddSection(const char * Name, sSection *& Head);
	eLineType		CheckLineType(const char * Line);

};

#endif // !defined(AFX_SETTINGS_H__C166045D_B3B7_41B0_BB90_D76C37807644__INCLUDED_)
