/****************************************************************************\

PSOProxy - http proxy server for the PSO web browser

Copyright (C) 2003 Adrian O'Grady and James Sutherland

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

\****************************************************************************/

// SysLog.cpp: implementation of the CSysLog class.
//
//////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>

#include "SysLog.h"

CSysLog				CSysLog::s_inst;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSysLog::CSysLog()
{
	m_pOutFile = 0;
}

CSysLog::~CSysLog()
{
	if (m_pOutFile)
	{
		StopLog();
	}
}

CSysLog& CSysLog::Get()
{
	return s_inst;
}

void CSysLog::StartLog(const char *Filename)
{
	time_t ltime;
	char buffer[128];

	if (m_pOutFile == 0)
	{
		m_pOutFile = new ofstream(Filename);

		time (&ltime);
		sprintf(buffer, "Log started: %s", ctime( &ltime ));
		__Log(buffer);
	}
	else
	{
		Log("CSysLog::StartLog - Logging already started.");
	}
}

void CSysLog::Log(const char *Text)
{
	char * Buffer = new char[strlen(Text) + 3];

	sprintf(Buffer, "%s\n", Text);
	__Log(Buffer);

	delete[] Buffer;
}

void CSysLog::StopLog()
{
	time_t ltime;
	char buffer[128];

	if (m_pOutFile)
	{
		time (&ltime);

		sprintf(buffer, "Log stopped: %s\n", ctime( &ltime ));
		__Log(buffer);
		
		TimeStamp(buffer);
		__Log(buffer);

		m_pOutFile->close();
		delete m_pOutFile;
		m_pOutFile = 0;
	}
}

void CSysLog::TimeStamp(char * Output)
{
    struct tm *newtime;
    time_t long_time;

    time( &long_time );                /* Get time as long integer. */
    newtime = localtime( &long_time ); /* Convert to local time. */

	sprintf(Output, "%02d-%02d-%d %02d:%02d:%02d", newtime->tm_mday,
										 newtime->tm_mon+1,
										 newtime->tm_year+1900,
										 newtime->tm_hour,
										 newtime->tm_min,
										 newtime->tm_sec);
}

void CSysLog::__Log(const char *Text)
{
	if (m_pOutFile)
	{
		m_pOutFile->write(Text, strlen(Text));
	}
}
