// VirtualPathMan.cpp: implementation of the CVirtualPathMan class.
//
//////////////////////////////////////////////////////////////////////

#include <string.h>

#include "VirtualPathMan.h"
#include "Settings.h"
#include "AppletDB.H"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CVirtualPathMan CVirtualPathMan::s_inst;

CVirtualPathMan::CVirtualPathMan()
{
	m_NumVirtualPaths = 0;
}

CVirtualPathMan::~CVirtualPathMan()
{

}

char * CVirtualPathMan::GetRealPath(const char *Path, char *Output)
{
	const char * VPath = CSettings::Get().GetValueAsString("vfile", Path);
	
	// First check if it's a virtual directory
	if (VPath == 0)
	{
		VPath = Path;
	}

	// Now see what type of path we have now
	switch (VPath[0])
	{
	case '!':
		// A real path
		strcpy(Output, &VPath[1]);
		break;

	default:

		if (CAppletDB::Get().IsApplet(VPath))
		{
			// It was an applet
			strcpy(Output, VPath);
		}
		else
		{
			// A normally requested file, build the full path
			GetRealVirtualPath(VPath, Output);
			if (Output[0] == 0)
			{
				strcpy(Output, CSettings::Get().GetValueAsString("server", "webroot", "webroot/"));
				if (VPath[0] == '/')
				{
					strcat(Output, &VPath[1]);
				}
				else
				{
					strcat(Output, VPath);
				}
			}
		}
		break;
	}

	return Output;
}

void CVirtualPathMan::OnDataItem(const char * Key, const char * Value, bool LastItem)
{
	int x = 0;
	if (!LastItem)
	{
		strcpy(m_PathList[m_NumVirtualPaths].VirtualPath, Key);
		x = strlen(m_PathList[m_NumVirtualPaths].VirtualPath) - 1;
		if (m_PathList[m_NumVirtualPaths].VirtualPath[x] != '/')
		{
			strcat(m_PathList[m_NumVirtualPaths].VirtualPath, "/");
		}

		strcpy(m_PathList[m_NumVirtualPaths].RealPath, Value);
		x = strlen(m_PathList[m_NumVirtualPaths].RealPath) - 1;
		if (m_PathList[m_NumVirtualPaths].RealPath[x] != '/')
		{
			strcat(m_PathList[m_NumVirtualPaths].RealPath, "/");
		}

		m_NumVirtualPaths++;
	}
}

void CVirtualPathMan::LoadVirtualPaths()
{
	CSettings::Get().GetAllSettings(this, "vdir");
}

char * CVirtualPathMan::GetRealVirtualPath(const char *Path, char *Output)
{
	char TempChar;
	char TempPath[512];
	int PathLen = strlen(Path);
	int VPathLen;

	strcpy(TempPath, Path);
	Output[0] = 0;

	for (int x = 0; x < m_NumVirtualPaths; x++)
	{
		VPathLen = strlen(m_PathList[x].VirtualPath);
		if (VPathLen <= PathLen)
		{
			TempChar = TempPath[VPathLen];
			TempPath[VPathLen] = 0;
			if (strcmp(TempPath, m_PathList[x].VirtualPath) == 0)
			{
				strcpy(Output, m_PathList[x].RealPath);
				strcat(Output, &Path[VPathLen]);
			}
			else
			{
				TempPath[VPathLen] = TempChar;
			}
		}
	}

	return Output;
}
