/****************************************************************************\

PSOProxy - http proxy server for the PSO web browser

Copyright (C) 2003 Adrian O'Grady and James Sutherland

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

\****************************************************************************/

// AppletDB.cpp: implementation of the CAppletDB class.
//
//////////////////////////////////////////////////////////////////////
#include <stdio.h>

#include "AppletDB.h"
#include "Settings.h"
#include "SysLog.h"

#include "PageServer.h"
#include "SaveFile.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAppletDB CAppletDB::s_inst;

CAppletDB::CAppletDB()
{
	m_AppletList = 0;
	m_AppletCount = 0;
}

CAppletDB::~CAppletDB()
{
	if (m_AppletList) delete[] m_AppletList;
}

CAppletDB::sAppletCallbacks * CAppletDB::GetApplet(const char *Name)
{
	sAppletCallbacks * r = 0;

	for (unsigned int i = 0; i < m_AppletCount; i++)
	{
		if (strcmp(m_AppletList[i].AppletName, Name) == 0)
		{
			r = &m_AppletList[i];
			break;
		}
	}

	return r;
}

int CAppletDB::ExecApplet(const char *Name, CAppletDB::AppletArgs Args)
{
	int r = -999;
	char Error[256];

	sAppletCallbacks * cb = GetApplet(Name);
	if (cb)
	{
		CProxyAPI1 NewAPI;
		strcpy(NewAPI.m_AppName, Name);
		NewAPI.m_BufInputLen = Args.BufferedInputLength;
		NewAPI.m_Socket = Args.Socket;
		NewAPI.m_Header = Args.Header;
		NewAPI.m_Parent = 0;
		if (NewAPI.m_BufInputLen)
		{
			NewAPI.m_BufInput = new char[NewAPI.m_BufInputLen];
			memcpy(NewAPI.m_BufInput, Args.BufferedInput, NewAPI.m_BufInputLen);
		}

		IProxyApplet1 * app = cb->CreateApplet(NewAPI);
		if (app)
		{
			sprintf(Error, "Executing %s", Name);
			CSysLog::Get().Log(Error);

			NewAPI.SetOwner(app);
			r = app->appmain(Args.argc, Args.argv);
			NewAPI.SetOwner(0);
			cb->ReleaseApplet(app);
		}
		else
		{
			sprintf(Error, "Unable to create an instance of %s", Name);
			CSysLog::Get().Log(Error);
		}

		if (NewAPI.m_BufInputLen)
		{
			delete[] NewAPI.m_BufInput;
			NewAPI.m_BufInput = 0;
		}
	}
	else
	{
		sprintf(Error, "Unable to find applet %s", Name);
		CSysLog::Get().Log(Error);
	}

	return r;
}

int CAppletDB::ExecApplet(const char *AppletName, const char *Args, CProxyAPI1 *Parent)
{
	char Error[256];
	int r = -999;
	CProxyAPI1 NewAPI;

	strcpy(NewAPI.m_AppName, AppletName);
	NewAPI.m_Parent = Parent;
	NewAPI.m_Header = Parent->m_Header;

	sAppletCallbacks * cb = GetApplet(AppletName);
	if (cb)
	{
		IProxyApplet1 * app = cb->CreateApplet(NewAPI);
		if (app)
		{
			sprintf(Error, "Executing %s", AppletName);
			CSysLog::Get().Log(Error);
			
			NewAPI.SetOwner(app);
			r = app->appmain(1, &Args);
			NewAPI.SetOwner(0);
			cb->ReleaseApplet(app);
		}
		else
		{
			sprintf(Error, "Unable to create an instance of %s", AppletName);
			CSysLog::Get().Log(Error);
		}
	
	}
	else
	{
		sprintf(Error, "Unable to find applet %s", AppletName);
		CSysLog::Get().Log(Error);
	}

	return r;
}

void CAppletDB::LoadDB()
{
	unsigned int Count = 0;

	m_AppletCount = CSettings::Get().GetValueAsInt("Applets", "count", 0);
	// Add space for our two internal applets
	m_AppletCount += 2;

	m_AppletList = new sAppletCallbacks[m_AppletCount];

	for (Count = 0; Count < m_AppletCount; Count++)
	{
		m_AppletList[Count].AppletName[0] = 0;
		m_AppletList[Count].CreateApplet  = 0;
		m_AppletList[Count].ReleaseApplet = 0;
	}

	Count = 0;
	// *** Load Applets ***//

	// Load internal applets
	strcpy(m_AppletList[Count].AppletName, "server");
	m_AppletList[Count].CreateApplet = CPageServer::CreateApplet;
	m_AppletList[Count].ReleaseApplet = CPageServer::ReleaseApplet;
	Count++;
	
	strcpy(m_AppletList[Count].AppletName, "snapshot");
	m_AppletList[Count].CreateApplet = CSaveFile::CreateApplet;
	m_AppletList[Count].ReleaseApplet = CSaveFile::ReleaseApplet;
	Count++;
}

bool CAppletDB::IsApplet(const char *AppletName)
{
	if (GetApplet(AppletName))
	{
		return true;
	}
	return false;
}
