/****************************************************************************\

PSOProxy - http proxy server for the PSO web browser

Copyright (C) 2003 Adrian O'Grady and James Sutherland

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

\****************************************************************************/

#include <winsock.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "resource.h"

#include "SysLog.h"
#include "RequestHandler.h"
#include "Settings.h"
#include "AppletDB.h"
#include "ProcessData.h"
#include "VirtualPathMan.h"

#define		PROXY_MAJOR				0
#define		PROXY_MINOR				9
#define		PROXY_REVISION			0

#define		NET_READ_BUFFER_SIZE	1504
#define		NET_READ_SIZE			1500

BOOL CALLBACK DialogFunc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

void OnStartUp(HWND hwnd);
void OnStartServer(HWND hwnd);
void OnStopServer(HWND hwnd);
void OnAbout(HWND hwnd);
void OnBrowse(HWND hwnd);

int InitWinsock();

DWORD WINAPI tProxyServer(void * Port);
DWORD WINAPI tConnection(void * AcceptSocket);

const char * __GetServerVersionString();
void __GetServerVersion(int& Major, int& Minor, int& Revision);

// Error messages
const char * GetGeneralSocketError(int socketError);
const char * GetAcceptError(int socketError);

HINSTANCE	g_hInstance;
HICON		g_hIcon;				// Icon for the window


bool		g_Quit = false;
SOCKET		g_ServerSock = 0;

static char s_WSA_ErrorString[1024];
static char s_AppTitle[63];

static CProcessData	s_oProcess;

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	int r = 0;

	CSettings::Get().LoadIni("proxy.ini");
	CAppletDB::Get().LoadDB();
	CVirtualPathMan::Get().LoadVirtualPaths();

	CProxyAPI1::StaticInitialise();

	g_hInstance		= hInstance;
	g_hIcon			= LoadIcon(hInstance, "ICON");
	
	r = DialogBox(hInstance, "IDD_MAIN", NULL, DialogFunc);

	CProxyAPI1::StaticDeinitialise();

	return r;
}

// The main message loop
BOOL CALLBACK DialogFunc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{

	switch(uMsg){
	case WM_INITDIALOG:
		OnStartUp(hwndDlg);
		break;

	// Button clicks
	case WM_COMMAND:
		switch(LOWORD(wParam)){
		// UDP echo server start
		case IDC_CMDSTART:
			OnStartServer(hwndDlg);
			break;

		case IDC_CMDSTOP:
			OnStopServer(hwndDlg);
			break;

		case IDC_CMDABOUT:
			OnAbout(hwndDlg);
			break;

		case IDC_CMDBROWSE:
			OnBrowse(hwndDlg);
			break;
		}	
		return 1;

	case WM_DROPFILES:
	{
		HDROP hDrop=(HDROP)wParam;
		int numFiles=DragQueryFile(hDrop, 0xFFFFFFFF, NULL, 0);
		char pFileName[MAX_PATH];

		//CProcessData oProcess;

		for(int i=0; i<numFiles; i++)
		{
			DragQueryFile(hDrop, i, pFileName, MAX_PATH);
			s_oProcess.OnFile(pFileName);
		}

		DragFinish(hDrop);

		return 1;
	}

	case WM_CLOSE:
		OnStopServer(hwndDlg);
		CSysLog::Get().StopLog();
		CSettings::Get().SaveIni();

		EndDialog(hwndDlg, 0);
		return 1;
	}

	return 0;
}

void OnStartUp(HWND hwnd)
{
	g_Quit = false;
	
	InitWinsock();

	CSysLog::Get().StartLog("log.txt");

	SetDlgItemText(hwnd, IDC_PORT,  CSettings::Get().GetValueAsString("server", "port", "8080"));
	SendMessage(hwnd, WM_SETICON, (WPARAM)ICON_BIG, (LPARAM)g_hIcon);

	LoadString(g_hInstance, IDS_APP_TITLE, s_AppTitle, 1024);
}

void OnStartServer(HWND hwnd)
{
	DWORD dwTid; // Again, a throw away value. Damn you Win95!
	unsigned short Port;
	char Buffer[256];

	EnableWindow(GetDlgItem(hwnd, IDC_PORT), FALSE);
	EnableWindow(GetDlgItem(hwnd, IDC_CMDSTART), FALSE);
	EnableWindow(GetDlgItem(hwnd, IDC_CMDSTOP), TRUE);

	g_Quit = false;

	GetDlgItemText(hwnd, IDC_PORT, Buffer, 5);
	Port = (unsigned short)atoi(Buffer);
	
	if (Port != (unsigned short)CSettings::Get().GetValueAsInt("server", "port", 8080))
	{
		CSettings::Get().SetValue("server", "port", (int)Port);
	}

	// The start page
	GetDlgItemText(hwnd, IDC_START, Buffer, 255);

	CreateThread(NULL, 0, tProxyServer, (void *)Port, 0, &dwTid);
}

void OnStopServer(HWND hwnd)
{
	EnableWindow(GetDlgItem(hwnd, IDC_PORT), TRUE);
	EnableWindow(GetDlgItem(hwnd, IDC_CMDSTART), TRUE);
	EnableWindow(GetDlgItem(hwnd, IDC_CMDSTOP), FALSE);

	g_Quit = true;
	closesocket(g_ServerSock);
	g_ServerSock = 0;
}

void OnAbout(HWND hwnd)
{
	TCHAR lpString[1024];
	LoadString(g_hInstance, IDS_ABOUT, lpString, 1024);
	MessageBox(hwnd, lpString, "About PSOProxy", MB_OK|MB_ICONINFORMATION);
}

void OnBrowse(HWND hwnd)
{
	char folder[MAX_PATH];
	GetCurrentDirectory(MAX_PATH, folder);
	strcat(folder, "\\");
	strcat(folder, CSettings::Get().GetValueAsString("snapshot", "dir"));
	if(folder[strlen(folder)-1]=='/') folder[strlen(folder)-1]='\0';
	ShellExecute(hwnd, "open", folder, NULL, folder, SW_SHOW);
}

// Initialise version 2.0 of the winsock library
int InitWinsock()
{
	WSADATA wsadata;
	int RetVal;
	WORD wVer;
	
	// Select the version and test it
	wVer = MAKEWORD(2,0);
	RetVal = WSAStartup(wVer, &wsadata);
	if(wsadata.wVersion != wVer) return -1;

	return 0;
}


DWORD WINAPI tProxyServer(void * Port)
{
	int RetVal;				// Return value
	sockaddr_in addr_serv;	// Local socket address
	SOCKET sdAccept;		// Socket that we'll be accepting
	DWORD dwTid;			// A throw away value, again, threads are fire and forget
	char ErrorString[256];	// For getting the string from the string table
	char Error[1024];		// For building error messages

	// Create the streaming server socket
	g_ServerSock = socket(PF_INET, SOCK_STREAM, 0);
	if(g_ServerSock == INVALID_SOCKET)
	{

		LoadString(g_hInstance, IDS_SOCKET_ERROR, ErrorString, 1024);
		MessageBox(0, ErrorString, s_AppTitle, MB_ICONEXCLAMATION);
		//WSAGetLastError();
		return -1;

	}

	// Set up the local address
	addr_serv.sin_family = PF_INET;
	addr_serv.sin_addr.s_addr = htonl(INADDR_ANY);
	addr_serv.sin_port = htons((unsigned short)Port);

	// Bind our server socket to the local address
	RetVal = bind(g_ServerSock, (sockaddr FAR *)&addr_serv, sizeof(addr_serv));
	if(RetVal == SOCKET_ERROR)
	{

		LoadString(g_hInstance, IDS_BIND_ERROR, ErrorString, 512);
		sprintf(Error, ErrorString, Port);
		CSysLog::Get().Log(Error);
		MessageBox(0, Error, s_AppTitle, MB_ICONEXCLAMATION);
		return -1;
	}

	// Start listening on our server socket
	RetVal = listen(g_ServerSock, SOMAXCONN);
	if(RetVal == SOCKET_ERROR)
	{
		LoadString(g_hInstance, IDS_LISTEN_ERROR, ErrorString, 512);
		CSysLog::Get().Log(ErrorString);
		MessageBox(0, ErrorString, s_AppTitle, MB_ICONEXCLAMATION);
		return -1;
	}

	// Loop until we're told to quit by the application
	while(g_Quit == false){
		// Wait until we have a connection request
		sdAccept = accept(g_ServerSock, NULL, NULL);
		if(g_Quit) break; // If the main application has told us to stop, break out of the loop

		// Make sure we're able to accept the socket
		if(sdAccept == INVALID_SOCKET)
		{
			LoadString(g_hInstance, IDS_ACCEPT_ERROR, ErrorString, 512);
			sprintf(Error, ErrorString, GetAcceptError(WSAGetLastError()));
			CSysLog::Get().Log(Error);
			MessageBox(0, Error, s_AppTitle, MB_ICONEXCLAMATION);
			return -1;
		}

		// Launch a thread to deal with the new connection and loop round again
		if (CreateThread(NULL, 0, tConnection, (void *)sdAccept, 0, &dwTid) == NULL)
		{
			// Couldn't launch the thread!
			LoadString(g_hInstance, IDS_THREAD_ERROR, ErrorString, 512);
			MessageBox(0, ErrorString, s_AppTitle, MB_ICONEXCLAMATION);
			CSysLog::Get().Log(ErrorString);

			g_Quit = true;
		}
	}

	closesocket(g_ServerSock);

	return 0;
}


DWORD WINAPI tConnection(void * socket)
{
	char InData[NET_READ_BUFFER_SIZE];		// Data read from the client
	bool Quit = false;						// Quit the read header loop
	int  DataSize = 0;						// Amount of data received
	SOCKET sock = (SOCKET)socket;			// The socket we've just accepted
	CRequestHandler	Request;				// Handles our input for us

	while (!Quit)
	{
		// Read the request header from the client
		DataSize = recv(sock, InData, NET_READ_SIZE, 0);
		if (DataSize < 1)
		{
			// No more data, time to move on
			Quit = true;
		}
		else
		{
			// Stream the data, um, somewhere...
			if (Request.StreamIn(InData, DataSize, sock) == CRequestHandler::ACT_CLOSE)
			{
				// We've been told to close the connection
				Quit = true;
			}
		} // if (DataSize == 0)

	} // while (!Quit)

	// Everything's finished, close the serving socket
	closesocket(sock);

	return 0;
}

const char * __GetServerVersionString()
{
	return s_AppTitle;
}

void __GetServerVersion(int& Major, int& Minor, int& Revision)
{
	Major = PROXY_MAJOR;
	Minor = PROXY_MINOR;
	Revision = PROXY_REVISION;
}

// Socket errors
const char * GetGeneralSocketError(int socketError)
{
	const char * r = s_WSA_ErrorString;

	switch(socketError)
	{
	case WSANOTINITIALISED:
		LoadString(g_hInstance, IDS_WSAERROR_001, s_WSA_ErrorString, 1024);
		break;


	case WSAENETDOWN:
		LoadString(g_hInstance, IDS_WSAERROR_002, s_WSA_ErrorString, 1024);
		break;

	case WSAEINTR:
		LoadString(g_hInstance, IDS_WSAERROR_003, s_WSA_ErrorString, 1024);
		break;

	case WSAEINPROGRESS:
		LoadString(g_hInstance, IDS_WSAERROR_004, s_WSA_ErrorString, 1024);
		break;

	case WSAENOBUFS:
		LoadString(g_hInstance, IDS_WSAERROR_005, s_WSA_ErrorString, 1024);
		break;

	case WSAENOTSOCK:
		LoadString(g_hInstance, IDS_WSAERROR_006, s_WSA_ErrorString, 1024);
		break;

	case WSAEOPNOTSUPP:
		LoadString(g_hInstance, IDS_WSAERROR_007, s_WSA_ErrorString, 1024);
		break;

	default:
		LoadString(g_hInstance, IDS_WSAERROR_UNKNOWN, s_WSA_ErrorString, 1024);
	}

	return r;
}
const char * GetAcceptError(int socketError)
{
	//** TO DO: Move into the string table

	GetGeneralSocketError(socketError);
	const char * r = s_WSA_ErrorString; // Already populated by now

	switch(socketError)
	{
	case WSAEINVAL:
		LoadString(g_hInstance, IDS_WSAERROR_008, s_WSA_ErrorString, 1024);
		break;

	case WSAEMFILE:
		LoadString(g_hInstance, IDS_WSAERROR_009, s_WSA_ErrorString, 1024);
		break;

	case WSAEWOULDBLOCK:
		LoadString(g_hInstance, IDS_WSAERROR_010, s_WSA_ErrorString, 1024);
		break;
	}

	return r;
}